local ACTION_SPELL_MISSED_DODGE = ACTION_SPELL_MISSED_DODGE
local ACTION_SPELL_MISSED_EVADE = ACTION_SPELL_MISSED_EVADE
local ChatFrame_AddMessageEventFilter = ChatFrame_AddMessageEventFilter
local ChatFrame_RemoveMessageEventFilter = ChatFrame_RemoveMessageEventFilter
local HP = HP
local ipairs = ipairs
local math_abs = math.abs
local next = next
local pairs = pairs
local table_wipe = table.wipe
local tonumber = tonumber
local type = type
local unpack = unpack

local addonName, addonData = ...
local L, print, module = addonData.L, addonData.print, addonData:NewModule("EXTRA_PETBATTLE", "PET_BATTLES")
local petBattle

local function ChatFilter(chatFrame, event, ...)
  if chatFrame == addonData:GetChatFrame() then
    return true
  end
end

function module:OnLoad()
  petBattle = {
    const = {
      ABILITY_DAMAGE = "damage",
      ABILITY_HEAL = "heal",
      AVOIDANCE_DODGE = "dodge",
      AVOIDANCE_MISS = "miss",
      TARGET_ENEMY = "enemy",
      TARGET_PET = "pet",
      TYPE_CRIT = 2,
      TYPE_NORMAL = 0,
      TYPE_STRONG = 1,
      TYPE_WEAK = -1,
    },

    matches = {
      [1] = {"^(.-) applied (.-) to enemy (.-) %(Critical%)%.$", function(self, source, effect, target)
        --self:EffectTo(source, effect, target == "team" and true or target, self.const.TYPE_CRIT, self.const.TARGET_ENEMY, 1)
        return 1 end},
      [2] = {"^(.-) applied (.-) to enemy (.-) %(Strong%)%.$", function(self, source, effect, target)
        --self:EffectTo(source, effect, target == "team" and true or target, self.const.TYPE_STRONG, self.const.TARGET_ENEMY, 1)
        return 1 end},
      [3] = {"^(.-) applied (.-) to enemy (.-) %(Weak%)%.$", function(self, source, effect, target)
        --self:EffectTo(source, effect, target == "team" and true or target, self.const.TYPE_WEAK, self.const.TARGET_ENEMY, 1)
        return 1 end},
      [4] = {"^(.-) applied (.-) to enemy (.-)%.$", function(self, source, effect, target)
        --self:EffectTo(source, effect, target == "team" and true or target, self.const.TYPE_NORMAL, self.const.TARGET_ENEMY, 1)
        return 1 end},
      [5] = {"^(.-) dealt (%d-) damage to enemy (.-) %(Critical%)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_CRIT, self.const.TARGET_ENEMY, self.const.ABILITY_DAMAGE)
        return 1 end},
      [6] = {"^(.-) dealt (%d-) damage to enemy (.-) %(Strong%)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_STRONG, self.const.TARGET_ENEMY, self.const.ABILITY_DAMAGE)
        return 1 end},
      [7] = {"^(.-) dealt (%d-) damage to enemy (.-) %(Weak%)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_WEAK, self.const.TARGET_ENEMY, self.const.ABILITY_DAMAGE)
        return 1 end},
      [8] = {"^(.-) dealt (%d-) damage to enemy (.-)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_NORMAL, self.const.TARGET_ENEMY, self.const.ABILITY_DAMAGE)
        return 1 end},
      [9] = {"^(.-) dealt (%d-) damage to your (.-) %(Critical%)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_CRIT, self.const.TARGET_PET, self.const.ABILITY_DAMAGE)
        return 1 end},
      [10] = {"^(.-) dealt (%d-) damage to your (.-) %(Strong%)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_STRONG, self.const.TARGET_PET, self.const.ABILITY_DAMAGE)
        return 1 end},
      [11] = {"^(.-) dealt (%d-) damage to your (.-) %(Weak%)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_WEAK, self.const.TARGET_PET, self.const.ABILITY_DAMAGE)
        return 1 end},
      [12] = {"^(.-) dealt (%d-) damage to your (.-)%.$", function(self, source, damage, target)
        self:DealtNumbersTo(source, damage, target == "team" and true or target, self.const.TYPE_NORMAL, self.const.TARGET_PET, self.const.ABILITY_DAMAGE)
        return 1 end},
      [13] = {"^(.-) fades from enemy (.-)%.$", function(self, effect, target)
        --self:EffectTo(nil, effect, target == "team" and true or target, self.const.TYPE_NORMAL, self.const.TARGET_ENEMY, nil)
        return 1 end},
      [14] = {"^(.-) fades from your (.-)%.$", function(self, effect, target)
        --self:EffectTo(nil, effect, target == "team" and true or target, self.const.TYPE_NORMAL, self.const.TARGET_PET, nil)
        return 1 end},
      [15] = {"^(.-) healed (%d-) damage from enemy (.-)%.$", function(self, effect, health, target)
        self:DealtNumbersTo(effect, health, target == "team" and true or target, self.const.TYPE_NORMAL, self.const.TARGET_ENEMY, self.const.ABILITY_HEAL)
        return 1 end},
      [16] = {"^(.-) healed (%d-) damage from your (.-)%.$", function(self, effect, health, target)
        self:DealtNumbersTo(effect, health, target == "team" and true or target, self.const.TYPE_NORMAL, self.const.TARGET_PET, self.const.ABILITY_HEAL)
        return 1 end},
      [17] = {"^(.-) is now enemy active pet%.$", function(self, target)
        self:ActiveEnemy(target)
        return 1 end},
      [18] = {"^(.-) is now your active pet%.$", function(self, target)
        self:ActivePet(target)
        return 1 end},
      [19] = {"^(.-) missed enemy (.-)%.$", function(self, ability, target)
        self:AvoidedTo(ability, target == "team" and true or target, self.const.TARGET_ENEMY, self.const.AVOIDANCE_MISS)
        return 1 end},
      [20] = {"^(.-) missed your (.-)%.$", function(self, ability, target)
        self:AvoidedTo(ability, target == "team" and true or target, self.const.TARGET_PET, self.const.AVOIDANCE_MISS)
        return 1 end},
      [21] = {"^(.-) was dodged by enemy (.-)%.$", function(self, ability, target)
        self:AvoidedTo(ability, target == "team" and true or target, self.const.TARGET_ENEMY, self.const.AVOIDANCE_DODGE)
        return 1 end},
      [22] = {"^(.-) was dodged by your (.-)%.$", function(self, ability, target)
        self:AvoidedTo(ability, target == "team" and true or target, self.const.TARGET_PET, self.const.AVOIDANCE_DODGE)
        return 1 end},
      [23] = {"^Enemy (.-) died%.$", function(self, target)
        self:DealtDeathTo(target, "enemy")
        return 1 end},
      [24] = {"^Your (.-) died%.$", function(self, target)
        self:DealtDeathTo(target, "pet")
        return 1 end},
      [25] = {"^Round (%d-)$", function(self, round)
        return 1 end},
      [26] = {"^Your (.-) gains (%d-) XP%.$", function(self, target, experience)
        self:GainedXP(target, experience)
        return 1 end},
      [27] = {"^Your (.-) has reached Level (%d-)%!$", function(self, target, level)
        self:GainedLevel(target, level)
        return 1 end},
      [28] = {"^(.-) trapped enemy (.-)%.$", function(self, trap, target)
        self:Trapped(trap, target)
        return 1 end},
    },

    --[[
    EffectTo = function(self, source, effect, target2, status, receiver, nil1)
      if type(self.combat[receiver]) ~= "table" then
        self.combat[receiver] = {}
      end
      if target2 == true then
        -- TODO: NYI when it's done to the whole party -we need to figure out how to deal with the identification issue of who is who (API vs. chat messages)
      else
        if type(self.combat[receiver][target2]) ~= "table" then
          self.combat[receiver][target2] = {}
        end
        self.combat[receiver][target2][effect] = nil1
      end
    end,
    ]]

    DealtNumbersTo = function(self, source, number, target2, status, receiver, healOrDamage)
      if type(self.combat[receiver]) ~= "table" then
        self.combat[receiver] = {}
      end
      if target2 == true then
        -- TODO: NYI when it's done to the whole party -we need to figure out how to deal with the identification issue of who is who (API vs. chat messages)
      else
        if type(self.combat[receiver][target2]) ~= "table" then
          self.combat[receiver][target2] = {}
        end
        if type(self.combat[receiver][target2][healOrDamage]) ~= "number" then
          self.combat[receiver][target2][healOrDamage] = 0
        end
        self.combat[receiver][target2][healOrDamage] = self.combat[receiver][target2][healOrDamage] + (tonumber(number) or 0)
      end
    end,

    ActiveEnemy = function(self, target)
      self.combat.activeEnemy = target
      if self.combat.activePet and self.combat.activeEnemy then
        print("|cffFFCC00Your|r |cff55FF55"..self.combat.activePet.."|r |cffFFCC00versus|r |cffFF5555"..self.combat.activeEnemy.."|r|cffFFCC00.|r")
      end
    end,

    ActivePet = function(self, target)
       self.combat.activePet = target
        if self.combat.activePet and self.combat.activeEnemy then
          print("|cffFFCC00Your|r |cff55FF55"..self.combat.activePet.."|r |cffFFCC00versus|r |cffFF5555"..self.combat.activeEnemy.."|r|cffFFCC00.|r")
        end
    end,

    AvoidedTo = function(self, ability, target2, receiver, missOrDodge)
      if type(self.combat[receiver]) ~= "table" then
        self.combat[receiver] = {}
      end
      if target2 == true then
        -- TODO: NYI when it's done to the whole party -we need to figure out how to deal with the identification issue of who is who (API vs. chat messages)
      else
        if type(self.combat[receiver][target2]) ~= "table" then
          self.combat[receiver][target2] = {}
        end
        if type(self.combat[receiver][target2][missOrDodge]) ~= "number" then
          self.combat[receiver][target2][missOrDodge] = 0
        end
        self.combat[receiver][target2][missOrDodge] = self.combat[receiver][target2][missOrDodge] + 1
      end
    end,

    DealtDeathTo = function(self, target, receiver)
      if type(self.combat.death) ~= "table" then
        self.combat.death = {}
      end
      if type(self.combat.death[receiver]) ~= "table" then
        self.combat.death[receiver] = {}
      end
      self.combat.death[receiver][target] = 1
    end,

    GainedXP = function(self, target, experience)
      if type(self.combat.xp) ~= "table" then
        self.combat.xp = {}
      end
      if type(self.combat.xp[target]) ~= "number" then
        self.combat.xp[target] = 0
      end
      self.combat.xp[target] = self.combat.xp[target] + experience
    end,

    GainedLevel = function(self, target, level)
      if type(self.combat.level) ~= "table" then
        self.combat.level = {}
      end
      self.combat.level[target] = level -- TODO: what about when we have two pets with the same name?
    end,

    Trapped = function(self, trap, target)
      if type(self.combat.trapped) ~= "table" then
        self.combat.trapped = {}
      end
      self.combat.trapped[target] = trap
    end,

    round = 0,
    combat = {},

    NewRound = function(self, isFirst)
      if isFirst then
        self.round = 0
      else
        self.round = self.round + 1
      end
      table_wipe(self.combat)
    end,

    EndCombat = function(self)
      if type(self.combat) == "table" then
        if type(self.combat.trapped) == "table" then
          local temp = ""
          for target, trap in pairs(self.combat.trapped) do
            temp = temp..target.." "
          end
          if temp:len() > 1 then
            print("|cff55FF55You have caught "..temp:sub(1, temp:len() - 1).."!|r")
          end
        end
        local temp = {}
        if type(self.combat.level) == "table" then
          for target, level in pairs(self.combat.level) do
            temp[target] = level
          end
        end
        if type(self.combat.xp) == "table" then
          for target, xp in pairs(self.combat.xp) do
            if temp[target] then
              print("|cff55FF55"..target.." gained "..xp.." XP and dinged level "..temp[target].."!|r")
              temp[target] = nil
            else
              print("|cff55FF55"..target.." gained "..xp.." XP!|r")
            end
          end
        end
      end
      self.round = 0
      table_wipe(self.combat)
    end,

    CombatLog = function(self, message)
      local data = nil
      for _, matchFunc in ipairs(self.matches) do
        data = {message:match(matchFunc[1])}
        if next(data) then
          return matchFunc[2](self, unpack(data))
        end
      end
    end,

    PrintRoundSummary = function(self, isOpening)
      if isOpening then return end
      local receivers, numberTypes, avoidanceTypes = {self.const.TARGET_PET, self.const.TARGET_ENEMY}, {self.const.ABILITY_DAMAGE, self.const.ABILITY_HEAL}, {self.const.AVOIDANCE_MISS, self.const.AVOIDANCE_DODGE}
      local temp, damage, heal, dodge, miss, hpdiff
      for _, receiver in ipairs(receivers) do
        if type(self.combat[receiver]) == "table" then
          for target, targetData in pairs(self.combat[receiver]) do
            temp, damage, heal, dodge, miss = "", 0, 0, 0, 0
            for _, numberType in ipairs(numberTypes) do
              if numberType == self.const.ABILITY_DAMAGE then
                damage = damage + (self.combat[receiver][target][numberType] or 0)
              elseif numberType == self.const.ABILITY_HEAL then
                heal = heal + (self.combat[receiver][target][numberType] or 0)
              end
            end
            for _, avoidanceType in ipairs(avoidanceTypes) do
              if avoidanceType == self.const.AVOIDANCE_MISS then
                miss = miss + (self.combat[receiver][target][avoidanceType] or 0)
              elseif avoidanceType == self.const.AVOIDANCE_DODGE then
                dodge = dodge + (self.combat[receiver][target][avoidanceType] or 0)
              end
            end
            hpdiff = damage - heal
            if hpdiff < 0 then
              temp = temp.."|cff55FF55"..math_abs(hpdiff)..HP.."|r "
            elseif hpdiff > 0 then
              temp = temp.."|cffFF5555"..hpdiff..HP.."|r "
            end
            if dodge > 0 then
              temp = temp.."|cffCCCCCC"..dodge.." "..ACTION_SPELL_MISSED_DODGE.."|r "
            end
            if miss > 0 then
              temp = temp.."|cffCCCCCC"..miss.." "..ACTION_SPELL_MISSED_EVADE.."|r "
            end
            if temp:len() > 1 then
              print("|cff999999R"..self.round.."|r |cff"..(receiver == self.const.TARGET_PET and "55FF55" or (receiver == self.const.TARGET_ENEMY and "FF5555" or "FFFFFF"))..target.."|r "..temp:sub(1, temp:len() - 1))
            end
          end
        end
      end
      if type(self.combat.death) == "table" then
        for _, receiver in ipairs(receivers) do
          if type(self.combat.death[receiver]) == "table" then
            for target, _ in pairs(self.combat.death[receiver]) do
              print("|cff999999R"..self.round.."|r |cff"..(receiver == self.const.TARGET_PET and "55FF55" or (receiver == self.const.TARGET_ENEMY and "FF5555" or "FFFFFF"))..target.."|r died.")
            end
          end
        end
      end
    end,
  }
end

function module:Enable()
  if not module.IsEnabled then
    module.IsEnabled = 1
  else
    return
  end

  ChatFrame_AddMessageEventFilter("CHAT_MSG_PET_BATTLE_COMBAT_LOG", ChatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_PET_BATTLE_INFO", ChatFilter)

  module:RegisterEvent("CHAT_MSG_PET_BATTLE_COMBAT_LOG")
  module:RegisterEvent("CHAT_MSG_PET_BATTLE_INFO")
  module:RegisterEvent("PET_BATTLE_CLOSE")
  module:RegisterEvent("PET_BATTLE_OPENING_START")
  module:RegisterEvent("PET_BATTLE_OVER")
  module:RegisterEvent("PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE")
end

function module:Disable()
  if module.IsEnabled then
    module.IsEnabled = nil
  else
    return
  end

  ChatFrame_RemoveMessageEventFilter("CHAT_MSG_PET_BATTLE_COMBAT_LOG", ChatFilter)
  ChatFrame_RemoveMessageEventFilter("CHAT_MSG_PET_BATTLE_INFO", ChatFilter)

  module:UnregisterEvent("CHAT_MSG_PET_BATTLE_COMBAT_LOG")
  module:UnregisterEvent("CHAT_MSG_PET_BATTLE_INFO")
  module:UnregisterEvent("PET_BATTLE_CLOSE")
  module:UnregisterEvent("PET_BATTLE_OPENING_START")
  module:UnregisterEvent("PET_BATTLE_OVER")
  module:UnregisterEvent("PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE")
end

function module:CHAT_MSG_PET_BATTLE_COMBAT_LOG(_, message)
  petBattle:CombatLog(message)
end

function module:CHAT_MSG_PET_BATTLE_INFO(_, message)
  petBattle:CombatLog(message)
end

function module:PET_BATTLE_CLOSE()
  petBattle.round = petBattle.round - 1 -- fix a issue if printing now (new round fires once too many)
  petBattle:PrintRoundSummary() -- any leftovers are printed now
  petBattle:EndCombat() -- fires twice due to double events, use close for the real ending, over is fired right after the last round ends
end

module.PET_BATTLE_OVER = module.PET_BATTLE_CLOSE

function module:PET_BATTLE_OPENING_START()
  petBattle:NewRound(true)
end

function module:PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE(_, message)
  petBattle:PrintRoundSummary(tonumber(message) == 0)
  petBattle:NewRound()
end
