﻿local _G = _G
local BONUS_ROLL_ROLLING = BONUS_ROLL_ROLLING
local ChatFrame_AddMessageEventFilter = ChatFrame_AddMessageEventFilter
local ChatFrame_RemoveMessageEventFilter = ChatFrame_RemoveMessageEventFilter
local ERR_QUEST_REWARD_EXP_I = ERR_QUEST_REWARD_EXP_I
local ERR_QUEST_REWARD_MONEY_S = ERR_QUEST_REWARD_MONEY_S
local format = format
local FRIENDS_FRIENDS_CHOICE_EVERYONE = FRIENDS_FRIENDS_CHOICE_EVERYONE
local GetCoinTextureString = GetCoinTextureString
local GetCurrencyLink = GetCurrencyLink
local GetCVarBool = GetCVarBool
local GetGuildInfo = GetGuildInfo
local GetItemQualityColor = GetItemQualityColor
local GetTime = GetTime
local GREED = GREED
local GUILD = GUILD
local HONOR = HONOR
local InCombatLockdown = InCombatLockdown
local ipairs = ipairs
local IsMasterLooter = IsMasterLooter
local IsModifiedClick = IsModifiedClick
local ItemRefTooltip = ItemRefTooltip
local LOOT_DISENCHANT_CREDIT = LOOT_DISENCHANT_CREDIT
local math_abs = math.abs
local NEED = NEED
local next = next
local pairs = pairs
local PASS = PASS
local ROLL_DISENCHANT = ROLL_DISENCHANT
local select = select
local SetItemRef = SetItemRef
local table_insert = table.insert
local table_sort = table.sort
local table_wipe = table.wipe
local tonumber = tonumber
local tostring = tostring
local type = type
local UnitName = UnitName
local XP = XP
local YOU_LOOT_MONEY = YOU_LOOT_MONEY

local addonName, addonData = ...
local L, print, module = addonData.L, addonData.print, addonData:NewModule("LOOT")
local matches, elapsed, report

local function NewLine(raw)
  if raw:sub(raw:len()) ~= "\n" then
    return raw.."\n"
  end
  return raw
end

local function OutputReport()
  local eventOrder = {
    "CHAT_MSG_COMBAT_XP_GAIN",
    "CHAT_MSG_COMBAT_GUILD_XP_GAIN",
    "CHAT_MSG_COMBAT_FACTION_CHANGE",
    "CHAT_MSG_COMBAT_HONOR_GAIN",
    "CHAT_MSG_MONEY",
    "CHAT_MSG_CURRENCY",
    "CHAT_MSG_LOOT",
  }

  local eventReport, temp
  local str, lastName = ""

  for _, event in ipairs(eventOrder) do
    eventReport = report[event]

    if eventReport then
      if event == "CHAT_MSG_COMBAT_XP_GAIN" then
        temp = eventReport[UnitName("player")]

        if temp then
          str = str .. format("|cffFFFF00%s: %d|r ", XP, temp)
        end
        
      elseif event == "CHAT_MSG_COMBAT_GUILD_XP_GAIN" then
        temp = eventReport[GUILD] or eventReport[GetGuildInfo("player")]

        if temp then
          str = str .. format("|cffFFFF00%s%s: %d|r ", GUILD:sub(1, 1):len() > 0 and GUILD:sub(1, 1) or GUILD:sub(1, 2), XP, temp)
        end

      elseif event == "CHAT_MSG_COMBAT_FACTION_CHANGE" then
        for factionName, reputation in pairs(eventReport) do
          if reputation > 0 then
            str = str .. format("|cff9696FF%s|r|cff%s+%d|r ", addonData:Abbreviate(factionName), "00FF00", reputation)
          elseif reputation < 0 then
            str = str .. format("|cff9696FF%s|r|cff%s-%d|r ", addonData:Abbreviate(factionName), "FF0000", math_abs(reputation))
          end
        end

      elseif event == "CHAT_MSG_COMBAT_HONOR_GAIN" then
        temp = select(2, next(eventReport))
        temp = format("%.2f", temp)
        if (tonumber(temp) or 0) > 0 then
          str = str .. format("|cff9696FF%s: %s|r ", HONOR, temp)
        end

      elseif event == "CHAT_MSG_CURRENCY" then
        for playerName, currencies in pairs(eventReport) do
          if lastName ~= playerName then
            str = str .. format("|cff%s%s|r|cffFFFF00:|r ", addonData:ClassColor(playerName), playerName)
            lastName = playerName
          end
          temp = 0
          if type(currencies) == "table" then
            for currencyLink, currencyCount in pairs(currencies) do
              if type(currencyCount) == "table" then
                currencyLink = currencyCount[1]
                currencyCount = tonumber(currencyCount[2]) or 1
              else
                currencyCount = tonumber(currencyCount) or 1
              end
              if addonData:GetBoolOpt("SHOW_LOOT_COUNT") then
                str = str .. format("%s%s%s ", addonData:GetIconWithLink(currencyLink, 1), currencyCount > 1 and "x"..currencyCount or "", addonData:GetCurrencyCount(currencyCount, currencyLink))
              else
                str = str .. format("%s%s ", addonData:GetIconWithLink(currencyLink, 1), currencyCount > 1 and "x"..currencyCount or "")
              end
              temp = temp + 1
            end
          end
          if temp == 0 then
            str = str .. format("|cff999999(%s)|r ", L.CHAT_NO_LOOT)
          end
        end

      elseif event == "CHAT_MSG_LOOT" then
        local sorted = {}
        for playerName, items in pairs(eventReport) do
          table_insert(sorted, {tostring(playerName), items})
        end
        table_sort(sorted, function(a, b) return a[1] < b[1] end)
        for i, data in ipairs(sorted) do
          local playerName, items = data[1], data[2]
          if playerName == "1" then
            lastName = ""
            if type(items) == "table" then
              str = NewLine(str)
              for item, rollData in pairs(items) do
                if addonData:GetBoolOpt("SHOW_ROLL_ICONS") then
                  item = addonData:GetIconWithLink(item)
                end
                str = str .. format("|cff00AA00%s|r %s|cff00AA00:|r ", L.CHAT_ROLL, item) -- BUG: "Wurf un" but this string is used by options for the word "Roll"
                local sortedRolls = {}
                local needFlag, bonusFlag, hasBonus
                for rollPlayerName, roll in pairs(rollData) do
                  if roll[1] == 1 then
                    needFlag = 1
                  end
                  roll[3], hasBonus = addonData:PlayerName(rollPlayerName)
                  if hasBonus then
                    bonusFlag = 1
                  end
                  table_insert(sortedRolls, roll)
                end
                table_sort(sortedRolls, function(a, b) return tonumber(a[2]) > tonumber(b[2]) end)
                for _, roll in ipairs(sortedRolls) do
                  local rollType, rollNumber, rollPlayerName = roll[1], roll[2], roll[3]
                  local color, suffix
                  if rollPlayerName == UnitName("player") or (not needFlag or (needFlag and rollType == 1)) and (not bonusFlag or (bonusFlag and rollNumber > 100)) then
                    if rollType == 1 then
                      color, suffix = "33EE33", L.CHAT_ROLL_N
                    elseif rollType == 2 then
                      color, suffix = "CCCC33", L.CHAT_ROLL_G
                    elseif rollType == 3 then
                      color, suffix = "EE3333", L.CHAT_ROLL_D
                    elseif rollType == 4 then
                      color, suffix = "CCCCCC", L.CHAT_ROLL_P
                    end
                    if addonData:GetBoolOpt("SHOW_CLASS_COLORS") then
                      str = str .. format("|cff%s%s|r|cff%s:%s:%d|r ", addonData:ClassColor(rollPlayerName), rollPlayerName, color, suffix, rollNumber)
                    else
                      str = str .. format("|cff%s%s:%s:%d|r ", color, rollPlayerName, suffix, rollNumber)
                    end
                  end
                end
                str = NewLine(str)
              end
            end

          else
            if lastName ~= playerName then
              str = str .. format("|cff%s%s|r|cffFFFF00:|r ", addonData:ClassColor(playerName), playerName)
              lastName = playerName
            end
            temp = 0
            if type(items) == "table" then
              for _, item in pairs(items) do
                local itemLink, itemCount
                if type(item) == "table" then
                  itemLink = item[1]
                  itemCount = tonumber(item[2]) or 1
                else
                  itemLink = item
                  itemCount = 1
                end
                if playerName == UnitName("player") and addonData:GetBoolOpt("SHOW_LOOT_COUNT") then
                  str = str .. format("%s%s%s ", addonData:GetIconWithLink(itemLink), itemCount > 1 and "x"..itemCount or "", addonData:GetItemCount(itemCount, itemLink, true, true))
                else
                  str = str .. format("%s%s ", addonData:GetIconWithLink(itemLink), itemCount > 1 and "x"..itemCount or "")
                end
                temp = temp + 1
              end
            end
            if temp == 0 then
              str = str .. format("|cff999999(%s)|r ", L.CHAT_NO_LOOT)
            end
          end
        end

      elseif event == "CHAT_MSG_MONEY" then
        for playerName, copper in pairs(eventReport) do
          if (tonumber(copper) or 0) > 0 then
            if lastName ~= playerName then
              str = str .. format("|cff%s%s|r|cffFFFF00:|r ", addonData:ClassColor(playerName), playerName)
              lastName = playerName
            end
            str = str .. format("|cffFFFF00%s|r ", GetCoinTextureString(copper))
          end
        end
      end
    end
  end

  if str ~= "" then
    for _, line in ipairs({("\n"):split(str)}) do
      line = line:trim()
      if line ~= "" then
        print(line)
      end
    end
  end
end

local function OnUpdate(module, elapse)
  if InCombatLockdown() and addonData:GetOpt("SLEEP_AFTER_COMBAT") ~= -1 then
    return
  end
  if not module.lastEvent then
    return
  end
  elapsed = elapsed + elapse
  if GetTime() - module.lastEvent < addonData:GetOpt("SLEEP_BETWEEN_EVENTS") then
    return
  end
  if elapsed >= addonData:GetOpt("SLEEP_AFTER_COMBAT") then
    elapsed = 0
    module.lastEvent = nil
    module:SetScript("OnUpdate", nil)
    OutputReport()
    table_wipe(report)
  end
end

local function ChatEventFilter(chatFrame, event, ...)
  if chatFrame == addonData:GetChatFrame() then
    if not addonData:GetBoolOpt("PET_BATTLES") and event:match("PET_BATTLE") then
      return
    end
    return true
  end
end

local function SpecialChatEventFilter(chatFrame, event, message, ...)
  if chatFrame == addonData:GetChatFrame() then
    if message:match(addonData:FormatMatcher(ERR_QUEST_REWARD_MONEY_S)) then
      module:CHAT_MSG_MONEY("CHAT_MSG_MONEY", format(YOU_LOOT_MONEY, message), ...)
      return true
    elseif message:match(addonData:FormatMatcher(ERR_QUEST_REWARD_EXP_I)) then
      return true
    end
    local itemLink, byName = message:match(addonData:FormatMatcher(LOOT_DISENCHANT_CREDIT))
    if itemLink then
      return false, format(LOOT_DISENCHANT_CREDIT, addonData:GetIconWithLink(itemLink), byName), ...
    end
  end
end

function addonData:AddEventFilter(event)
  if not MiniLootDB.HIDE_EVENTS then
    MiniLootDB.HIDE_EVENTS = {}
  end
  MiniLootDB.HIDE_EVENTS[event] = 1
end

function addonData:RemoveEventFilter(event)
  if not MiniLootDB.HIDE_EVENTS then
    return
  end
  MiniLootDB.HIDE_EVENTS[event] = nil
end

function addonData:HasEventFilter(event)
  if not MiniLootDB.HIDE_EVENTS then
    return
  end
  return MiniLootDB.HIDE_EVENTS[event]
end

function module:OnLoad()
  matches = {
    CHAT_MSG_COMBAT_FACTION_CHANGE = { -- name(faction), value(rep), type(1/nil for inc/dec)
      [1] = {FACTION_STANDING_DECREASED = {1, 2, nil}},
      [2] = {FACTION_STANDING_DECREASED_GENERIC = {1, nil, nil}},
      [3] = {FACTION_STANDING_INCREASED = {1, 2, 1}},
      [4] = {FACTION_STANDING_INCREASED_BONUS = {1, 2, 1}},
      [5] = {FACTION_STANDING_INCREASED_GENERIC = {1, nil, 1}},
    },
    CHAT_MSG_COMBAT_HONOR_GAIN = { -- points(honor), source(name), rank(int)
      [1] = {COMBATLOG_HONORAWARD = {1, nil, nil}},
      [2] = {COMBATLOG_HONORGAIN = {3, 1, 2}},
      [3] = {COMBATLOG_HONORGAIN_NO_RANK = {2, 1, nil}},
    },
    CHAT_MSG_COMBAT_XP_GAIN = { -- points(exp), source(name), isLost(1/nil for lost or gained)
      [1] = {COMBATLOG_XPGAIN_EXHAUSTION1 = {2, 1, nil}},
      [2] = {COMBATLOG_XPGAIN_EXHAUSTION1_GROUP = {2, 1, nil}},
      [3] = {COMBATLOG_XPGAIN_EXHAUSTION1_RAID = {2, 1, nil}},
      [4] = {COMBATLOG_XPGAIN_EXHAUSTION2 = {2, 1, nil}},
      [5] = {COMBATLOG_XPGAIN_EXHAUSTION2_GROUP = {2, 1, nil}},
      [6] = {COMBATLOG_XPGAIN_EXHAUSTION2_RAID = {2, 1, nil}},
      [7] = {COMBATLOG_XPGAIN_EXHAUSTION4 = {2, 1, nil}},
      [8] = {COMBATLOG_XPGAIN_EXHAUSTION4_GROUP = {2, 1, nil}},
      [9] = {COMBATLOG_XPGAIN_EXHAUSTION4_RAID = {2, 1, nil}},
      [10] = {COMBATLOG_XPGAIN_EXHAUSTION5 = {2, 1, nil}},
      [11] = {COMBATLOG_XPGAIN_EXHAUSTION5_GROUP = {2, 1, nil}},
      [12] = {COMBATLOG_XPGAIN_EXHAUSTION5_RAID = {2, 1, nil}},
      [13] = {COMBATLOG_XPGAIN_FIRSTPERSON = {2, 1, nil}},
      [14] = {COMBATLOG_XPGAIN_FIRSTPERSON_GROUP = {2, 1, nil}},
      [15] = {COMBATLOG_XPGAIN_FIRSTPERSON_RAID = {2, 1, nil}},
      [16] = {COMBATLOG_XPGAIN_FIRSTPERSON_UNNAMED = {1, nil, nil}},
      [17] = {COMBATLOG_XPGAIN_FIRSTPERSON_UNNAMED_GROUP = {1, nil, nil}},
      [18] = {COMBATLOG_XPGAIN_FIRSTPERSON_UNNAMED_RAID = {1, nil, nil}},
      [19] = {COMBATLOG_XPGAIN_QUEST = {1, nil, nil}},
      [20] = {COMBATLOG_XPLOSS_FIRSTPERSON_UNNAMED = {1, nil, 1}},
    },
    CHAT_MSG_CURRENCY = { -- name(item, no link), count(int), target(name)
      [1] = {CURRENCY_GAINED_MULTIPLE = {1, 2, nil}},
      [2] = {CURRENCY_GAINED = {1, nil, nil}},
    },
    CHAT_MSG_LOOT = { -- name(item), count(int), target(name)
      [1] = {LOOT_ITEM_CREATED_SELF_MULTIPLE = {1, 2, nil}},
      [2] = {LOOT_ITEM_CREATED_SELF = {1, nil, nil}},
      [3] = {LOOT_ITEM_PUSHED_SELF_MULTIPLE = {1, 2, nil}},
      [4] = {LOOT_ITEM_PUSHED_SELF = {1, nil, nil}},
      [5] = {LOOT_ITEM_SELF_MULTIPLE = {1, 2, nil}},
      [6] = {LOOT_ITEM_SELF = {1, nil, nil}},
      [7] = {LOOT_ITEM_MULTIPLE = {2, 3, 1}},
      [8] = {LOOT_ITEM = {2, nil, 1}},
      [9] = {CREATED_ITEM_MULTIPLE = {2, 3, 1}},
      [10] = {CREATED_ITEM = {2, nil, 1}},
      -- roll handling; item(name), player(name), roll(int), type(nil or 1:disenchant, 2:greed, 3:need, 4:lost), isInstant
      [11] = {LOOT_ROLL_YOU_WON_NO_SPAM_NEED = {3, nil, 2, 3, nil}},
      [12] = {LOOT_ROLL_YOU_WON_NO_SPAM_GREED = {3, nil, 2, 2, nil}},
      [13] = {LOOT_ROLL_YOU_WON_NO_SPAM_DE = {3, nil, 2, 1, nil}},
      [14] = {LOOT_ROLL_WON_NO_SPAM_NEED = {4, 2, 3, 3, nil}},
      [15] = {LOOT_ROLL_WON_NO_SPAM_GREED = {4, 2, 3, 2, nil}},
      [16] = {LOOT_ROLL_WON_NO_SPAM_DE = {4, 2, 3, 1, nil}},
      [17] = {LOOT_ROLL_ROLLED_NEED_ROLE_BONUS = {2, 3, 1, 3, nil}},
      [18] = {LOOT_ROLL_ROLLED_NEED = {2, 3, 1, 3, nil}},
      [19] = {LOOT_ROLL_ROLLED_GREED = {2, 3, 1, 2, nil}},
      [20] = {LOOT_ROLL_ROLLED_DE = {2, 3, 1, 1, nil}},
      [21] = {LOOT_ROLL_LOST_ROLL = {}}, -- ?
      -- instant roll decision info; item(name), player(name), type(nil:pass, 1:disenchant, 2:greed, 3:need, 4:allPass, 5:rollStarted, 6:won), dummy(nil), isInstant
      [22] = {LOOT_ROLL_YOU_WON = {1, nil, 6, nil, 1}},
      [23] = {LOOT_ROLL_WON = {2, 1, 6, nil, 1}},
      [24] = {LOOT_ROLL_ALL_PASSED = {2, nil, 4, nil, 1}},
      [25] = {LOOT_ROLL_PASSED_SELF_AUTO = {2, nil, nil, nil, 1}},
      [26] = {LOOT_ROLL_PASSED_SELF = {2, nil, nil, nil, 1}},
      [27] = {LOOT_ROLL_PASSED_AUTO_FEMALE = {2, 1, nil, nil, 1}},
      [28] = {LOOT_ROLL_PASSED_AUTO = {2, 1, nil, nil, 1}},
      [29] = {LOOT_ROLL_PASSED = {2, 1, nil, nil, 1}},
      [30] = {LOOT_ROLL_NEED_SELF = {2, nil, 3, nil, 1}},
      [31] = {LOOT_ROLL_NEED = {2, 1, 3, nil, 1}},
      [32] = {LOOT_ROLL_GREED_SELF = {2, nil, 2, nil, 1}},
      [33] = {LOOT_ROLL_GREED = {2, 1, 2, nil, 1}},
      [34] = {LOOT_ROLL_DISENCHANT_SELF = {2, nil, 1, nil, 1}},
      [35] = {LOOT_ROLL_DISENCHANT = {2, 1, 1, nil, 1}},
      -- roll started; item(name), rollId(int), isRollStart(5), dummy(nil), isInstant
      [36] = {LOOT_ROLL_STARTED = {2, 1, 5, nil, 1}},
    },
    CHAT_MSG_MONEY = { -- gold(money), target(name), unused(nil)
      [1] = {YOU_LOOT_MONEY_GUILD = {1, nil, nil}},
      [2] = {YOU_LOOT_MONEY = {1, nil, nil}},
      [3] = {LOOT_MONEY_SPLIT_GUILD = {1, nil, nil}},
      [4] = {LOOT_MONEY_SPLIT = {1, nil, nil}},
      [5] = {LOOT_MONEY = {2, 1, nil}},
    },
    CHAT_MSG_COMBAT_GUILD_XP_GAIN = { -- points(exp)
      [1] = {COMBATLOG_GUILD_XPGAIN = {1}},
    },
  }

  report = {}

  do
    local tempMatches, tempIndex, tempPattern = {}

    for event, patterns in pairs(matches) do
      for i, tablePattern in ipairs(patterns) do
        for pattern, data in pairs(tablePattern) do
          if not tempMatches[event] then
            tempMatches[event] = {}
          end
          tempIndex = #tempMatches[event] + 1
          if not tempMatches[event][tempIndex] then
            tempMatches[event][tempIndex] = {}
          end
          tempPattern = "^" .. addonData:FormatMatcher(_G[pattern] or pattern) .. "$"
          tempMatches[event][tempIndex][tempPattern] = data
        end
      end
    end

    table_wipe(matches)
    matches = tempMatches
  end

  do
    local _SetHyperlink = ItemRefTooltip.SetHyperlink
    local _SetItemRef = SetItemRef

    function ItemRefTooltip:SetHyperlink(link, ...)
      if module.IsEnabled and link:find(addonName, 0, 1) then
        return
      end
      return _SetHyperlink(self, link, ...)
    end

    function SetItemRef(link, text, button, chatFrame, ...)
      local curID = tonumber(link:match("currency:(%d+)"))
      if module.IsEnabled and curID and curID > 0 and IsModifiedClick() then
        text = GetCurrencyLink(curID) or text
      end
      return _SetItemRef(link, text, button, chatFrame, ...)
    end
  end
end

function module:Enable()
  if not module.IsEnabled then
    module.IsEnabled = 1
  else
    return
  end

  for event, _ in pairs(matches) do
    ChatFrame_AddMessageEventFilter(event, ChatEventFilter)
    module:RegisterEvent(event)
  end

  ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", SpecialChatEventFilter)

  module:RegisterEvent("LOOT_OPENED")
  module:RegisterEvent("LOOT_CLOSED")

  elapsed = 0
  --module:SetScript("OnUpdate", OnUpdate)
end

function module:Disable()
  if module.IsEnabled then
    module.IsEnabled = nil
  else
    return
  end

  for event, _ in pairs(matches) do
    ChatFrame_RemoveMessageEventFilter(event, ChatEventFilter)
    module:UnregisterEvent(event)
  end

  ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", SpecialChatEventFilter)

  module:UnregisterEvent("LOOT_OPENED")
  module:UnregisterEvent("LOOT_CLOSED")

  module:SetScript("OnUpdate", nil)
  elapsed = nil

  module.lastEvent = nil
  table_wipe(report)
end

function module:BUMP_LAST_EVENT()
  module.lastEvent = GetTime()
end

module.LOOT_OPENED = module.BUMP_LAST_EVENT
module.LOOT_CLOSED = module.BUMP_LAST_EVENT

function module:ON_MATCHER_EVENT(event, message, ...)
  module:BUMP_LAST_EVENT()

  local temp, found, ptable

  for i, tablePattern in ipairs(matches[event]) do
    for pattern, data in pairs(tablePattern) do
      temp = {message:match(pattern)}
      if #temp > 0 then
        found = temp
        ptable = data
        temp = pattern
        break
      end
    end
    if found then
      break
    end
  end

  if not found or not ptable then
    return
  end

  if addonData:HasEventFilter(event) then
    return
  end

  if not report[event] then
    report[event] = {}
  end

  --UIParentLoadAddOn("Blizzard_DebugTools") DevTools_Dump(temp) DevTools_Dump(found) DevTools_Dump(ptable) -- DEBUG

  local param1, param2, param3, param4, param5

  if event == "CHAT_MSG_COMBAT_FACTION_CHANGE" then
    param1 = ptable[1] and found[ptable[1]]

    if param1 then
      param2 = ptable[2] and tonumber(found[ptable[2]]) or 0
      param3 = ptable[3] and 1 or -1

      report[event][param1] = (report[event][param1] or 0) + (math_abs(param2) * param3)
    end

  elseif event == "CHAT_MSG_COMBAT_HONOR_GAIN" then
    param1 = ptable[1] and tonumber(found[ptable[1]])

    if param1 then
      --param2 = ptable[2] and found[ptable[2]] or ""
      --param3 = ptable[3] and tonumber(found[ptable[3]]) or 0

      report[event][1] = (report[event][1] or 0) + param1
    end

  elseif event == "CHAT_MSG_COMBAT_XP_GAIN" then
    param1 = ptable[1] and tonumber(found[ptable[1]])

    if param1 then
      param2 = UnitName("player") -- ptable[2] and found[ptable[2]] or "" -- contains name of unit that rewarded us the XP but no point using it in the report so we go by player name
      param3 = ptable[3] and -1 or 1

      report[event][param2] = (report[event][param2] or 0) + (math_abs(param1) * param3)
    end

  elseif event == "CHAT_MSG_CURRENCY" then
    param1 = ptable[1] and found[ptable[1]]

    if param1 then
      param2 = ptable[2] and tonumber(found[ptable[2]]) or 1
      param3 = ptable[3] and addonData:PlayerName(found[ptable[3]]) or UnitName("player")

      if not report[event][param3] then
        report[event][param3] = {}
      end

      report[event][param3][param1] = (report[event][param3][param1] or 0) + param2
    end

  elseif event == "CHAT_MSG_LOOT" then
    param1 = ptable[1] and found[ptable[1]]
    param4 = ptable[4]
    param5 = ptable[5]

    -- TODO: exception for when we have a itemlink containing an "x" in the name while matching for "[itemlink]x5" (quantity message)
    if param1 and param1:match("|h%[") and not param1:match("|h%[(.-)%]|h") then
      local countIndex = param4 and 3 or 2 -- rolls use 3 to store itemLink while regular loot uses 2 (param4 is the roll flag)
      if tostring(found[ptable[countIndex]]):match("%]|h") then
        local itemColor, itemData, itemName, itemCount = message:match("|cff(.+)|Hitem:(.+)|h%[(.+)%]|h|r(.+)")
        found[countIndex] = tonumber(itemCount:match("(%d+)")) or 1
        found[1] = format("|cff%s|Hitem:%s|h[%s]|h|r", itemColor, itemData, itemName)
        param1 = ptable[1] and found[ptable[1]]
      end
    end

    if param1 and param5 then
      if addonData:GetBoolOpt("SHOW_ROLL_DECISIONS") and addonData:InLFR(not addonData:GetBoolOpt("HIDE_LFR_ROLL_DECISIONS")) then
        param1 = addonData:GetBoolOpt("SHOW_ROLL_ICONS") and addonData:GetIconWithLink(param1) or param1
        param2 = ptable[2] and addonData:PlayerName(found[ptable[2]]) or UnitName("player")
        param3 = ptable[3]

        if param3 == 4 then
          print(format("%s |cff%s%s: %s|r ", param1, "CCCCCC", FRIENDS_FRIENDS_CHOICE_EVERYONE, PASS))
        elseif param3 == 5 then
          if not (GetCVarBool("autoOpenLootHistory") or IsMasterLooter()) then
            print(format("%s |cff%s|HlootHistory:%d|h[%s...]|h|r ", param1, "CCCCCC", param2, BONUS_ROLL_ROLLING))
          end
        elseif param3 == 6 then
          print(format("%s |cff%s%s %s|r ", param1, "CCCCCC", param2, L.CHAT_WON))
        else
          if addonData:GetBoolOpt("SHOW_CLASS_COLORS") then
            print(format("%s |cff%s%s|r|cff%s: %s|r ", param1, addonData:ClassColor(param2), param2, (param3 == 1 and "EE3333") or (param3 == 2 and "CCCC33") or (param3 == 3 and "33EE33") or "CCCCCC", (param3 == 1 and ROLL_DISENCHANT) or (param3 == 2 and GREED) or (param3 == 3 and NEED) or PASS))
          else
            print(format("%s |cff%s%s: %s|r ", param1, (param3 == 1 and "EE3333") or (param3 == 2 and "CCCC33") or (param3 == 3 and "33EE33") or "CCCCCC", param2, (param3 == 1 and ROLL_DISENCHANT) or (param3 == 2 and GREED) or (param3 == 3 and NEED) or PASS))
          end
        end
      end

    elseif param1 and param4 then
      if addonData:GetBoolOpt("SHOW_ROLL_SUMMARY") and addonData:InLFR(not addonData:GetBoolOpt("HIDE_LFR_ROLL_SUMMARY")) then
        param1 = addonData:GetBoolOpt("SHOW_ROLL_ICONS") and addonData:GetIconWithLink(param1) or param1
        param2 = ptable[2] and addonData:PlayerName(found[ptable[2]]) or UnitName("player")
        param3 = ptable[3] and tonumber(found[ptable[3]])
        param4 = tonumber(ptable[4])

        if not report[event][1] then
          report[event][1] = {}
        end

        if not report[event][1][param1] then
          report[event][1][param1] = {}
        end

        if param4 == 1 then
          report[event][1][param1][param2] = {3, param3}
        elseif param4 == 2 then
          report[event][1][param1][param2] = {2, param3}
        elseif param4 == 3 then
          report[event][1][param1][param2] = {1, param3}
        else
          report[event][1][param1][param2] = {0, 0}
        end
      end

    elseif param1 then
      param1 = addonData:GetBoolOpt("SHOW_ROLL_ICONS") and addonData:GetIconWithLink(param1) or param1

      if not addonData:GetBoolOpt("HIDE_JUNK") or not param1:find(select(4, GetItemQualityColor(0)), nil, 1) then
        param2 = ptable[2] and tonumber(found[ptable[2]]) or 1
        param3 = ptable[3] and addonData:PlayerName(found[ptable[3]]) or UnitName("player")

        local threshold, skip

        if addonData:GetBoolOpt("HIDE_PARTY_LOOT") and param3 ~= UnitName("player") then
          threshold = addonData:GetOpt("HIDE_PARTY_LOOT_T")
        elseif addonData:GetBoolOpt("HIDE_SOLO_LOOT") and param3 == UnitName("player") then
          threshold = addonData:GetOpt("HIDE_SOLO_LOOT_T")
        end

        if threshold and (threshold == 8 or threshold > addonData:GetQualityInfo(param1)) then
          if not addonData:ItemStartsQuest(param1, 1) then
            skip = 1
          end
        end

        if not skip then
          local itemID = tonumber(tostring(param1):match("item:(%d+)") or param1)

          if itemID then
            if not report[event][param3] then
              report[event][param3] = {}
            end

            if not report[event][param3][itemID] then
              report[event][param3][itemID] = {param1, param2, param3}
            else
              report[event][param3][itemID][2] = report[event][param3][itemID][2] + param2
            end
          else
            _G.print("ERROR: MiniLoot could not log regular item loot event with the following parameters: {"..event..", "..message..", "..tostring(param1)..", "..tostring(param2)..", "..tostring(param3)..", "..tostring(param4)..", "..tostring(param5).."} Please report this, and any other error messages that may have appeared, to the developers - thank you!") -- DEBUG: until this error is fixed, this helps users report more helpful comments!
          end
        end
      end
    end

  elseif event == "CHAT_MSG_MONEY" then
    param1 = ptable[1] and found[ptable[1]]

    if param1 then
      param2 = ptable[2] and addonData:PlayerName(found[ptable[2]]) or UnitName("player")

      report[event][param2] = (report[event][param2] or 0) + addonData:MoneyToCopper(param1)
    end

  elseif event == "CHAT_MSG_COMBAT_GUILD_XP_GAIN" then
    param1 = ptable[1] and tonumber(found[ptable[1]])

    if param1 then
      param2 = GetGuildInfo("player") or GUILD

      report[event][param2] = (report[event][param2] or 0) + math_abs(param1)
    end
  end

  module:SetScript("OnUpdate", OnUpdate)
end

module.CHAT_MSG_COMBAT_FACTION_CHANGE = module.ON_MATCHER_EVENT
module.CHAT_MSG_COMBAT_HONOR_GAIN = module.ON_MATCHER_EVENT
module.CHAT_MSG_COMBAT_XP_GAIN = module.ON_MATCHER_EVENT
module.CHAT_MSG_CURRENCY = module.ON_MATCHER_EVENT
module.CHAT_MSG_LOOT = module.ON_MATCHER_EVENT
module.CHAT_MSG_MONEY = module.ON_MATCHER_EVENT
module.CHAT_MSG_COMBAT_GUILD_XP_GAIN = module.ON_MATCHER_EVENT
